/**
 * Launcher.java
 *
 *
 * Created: Tue Jan 29 00:05:42 2002
 *
 * @author <a href="mailto:torutk@alles.or.jp">Toru TAKAHASHI</a>
 * @version
 */
import java.util.Observable;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.IOException;
import javax.swing.JOptionPane;
import java.util.Arrays;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.logging.Logger;

public class Launcher extends Observable {
    private List commands;
    private static final String CLASS_NAME = "Launcher";
    private static Logger logger = Logger.getLogger(CLASS_NAME);

    public Launcher () {
        commands = new ArrayList();
    }

    String getCommandText() {
        StringBuffer strBuf = new StringBuffer();
        for (Iterator it = commands.iterator();
             it.hasNext(); strBuf.append(' ')) {
            strBuf.append((String)it.next());
        }
        return strBuf.toString();
    }

    void setCommandText(String text) {
        logger.entering(CLASS_NAME, "setCommandText", text);
        StringTokenizer st = new StringTokenizer(text);
        String[] args = new String[st.countTokens()];
        for (int i=0; st.hasMoreTokens(); i++) {
            args[i] = st.nextToken();
        }
        setCommands(args);
    }

    void setCommands(String[] args) {
        logger.entering(CLASS_NAME, "setCommands", args);
        commands = Arrays.asList(args);
        setChanged();
        notifyObservers();
    }

    void launch() {
        String[] commandArgs = (String[])commands.toArray(new String[0]);
        Runtime runtime = Runtime.getRuntime();
        try {
            Process process = runtime.exec(commandArgs);
            final BufferedReader stdReader = new BufferedReader(
                new InputStreamReader(process.getInputStream())
            );
            Thread stdOutThread = new Thread(new Runnable() {
                    public void run() {
                        String line;
                        try {
                            while ((line = stdReader.readLine()) != null) {
                                System.out.println(line);
                            }
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                });
            stdOutThread.start();

            final BufferedReader errReader = new BufferedReader(
                new InputStreamReader(process.getErrorStream())
            );
            Thread stdErrThread = new Thread(new Runnable() {
                    public void run() {
                        String line;
                        try {
                            while ((line = errReader.readLine()) != null) {
                                System.err.println(line);
                            }
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                });
            stdErrThread.start();
        } catch (IOException e) {
            JOptionPane.showMessageDialog(
                null, "Command '" + getCommandText() + "' cannot execute.",
                "Launcher Error", JOptionPane.ERROR_MESSAGE
            );
        }                                          
    }
}// Launcher
